@@ -32,9 +32,11 @@
 #define CLIENT_UTIL_H
 
 #include "amanda.h"
+#include "conffile.h"
 #include "amfeatures.h"
 #include "sl.h"
 #include "util.h"		/* for bstrncmp() */
+#include "amandad.h"		/* for g_option_t */
 
 typedef struct option_s {
     char *str;
@@ -57,17 +59,24 @@ typedef struct option_s {
     int include_optional;
 } option_t;
 
-#define NO_COMPR   0
-#define COMPR_FAST 1
-#define COMPR_BEST 2
-#define COMPR_SERVER_FAST 3
-#define COMPR_SERVER_BEST 4
-#define COMPR_SERVER_CUST 5	/* server-side custom compression */
-#define COMPR_CUST 6            /* client-side custom compression */
-
-#define ENCRYPT_NONE         0	/* no encryption  */
-#define ENCRYPT_CUST         1	/* client-side custom encryption */
-#define ENCRYPT_SERV_CUST    2	/* server-side custom encryption */
+typedef struct backup_support_option_s {
+    int config;
+    int host;
+    int disk;
+    int max_level;
+    int index_line;
+    int index_xml;
+    int message_line;
+    int message_xml;
+    int record;
+    int include_file;
+    int include_list;
+    int include_optional;
+    int exclude_file;
+    int exclude_list;
+    int exclude_optional;
+    int collection;
+} backup_support_option_t;
 
 char *build_exclude(char *disk, char *device, option_t *options, int verbose);
 char *build_include(char *disk, char *device, option_t *options, int verbose);
@@ -77,7 +86,11 @@ option_t *parse_options(char *str,
 			   char *device,
 			   am_feature_t *features,
 			   int verbose);
-
+void output_tool_property(FILE *tool, option_t *options);
 char *fixup_relative(char *name, char *device);
+backup_support_option_t *backup_support_option(char *program,
+					       g_option_t *g_options,
+					       char *disk,
+					       char *amdevice);
 
 #endif
