@@ -64,13 +64,14 @@ typedef struct regex_s {
     char *regex;
     int srcline;
     int scale;                  /* only used for size lines */
+    int field;
     dmpline_t typ;
 } amregex_t;
 
-#define AM_NORMAL_RE(re)	{(re), __LINE__, 0, DMP_NORMAL}
-#define AM_STRANGE_RE(re)	{(re), __LINE__, 0, DMP_STRANGE}
-#define AM_SIZE_RE(re,s)	{(re), __LINE__, (s), DMP_SIZE}
-#define AM_ERROR_RE(re)		{(re), __LINE__, 0, DMP_ERROR}
+#define AM_NORMAL_RE(re)	{(re), __LINE__, 0, 0, DMP_NORMAL}
+#define AM_STRANGE_RE(re)	{(re), __LINE__, 0, 0, DMP_STRANGE}
+#define AM_SIZE_RE(re,s,f)	{(re), __LINE__, (s), (f), DMP_SIZE}
+#define AM_ERROR_RE(re)		{(re), __LINE__, 0, 0, DMP_ERROR}
 
 extern pid_t  comppid, dumppid, encpid, tarpid;
 extern pid_t indexpid;
