@@ -158,7 +158,12 @@ timesub(
 	end.tv_usec += 1000000;
     }
     diff.tv_usec = end.tv_usec - start.tv_usec;
-    diff.tv_sec = end.tv_sec - start.tv_sec;
+
+    if (end.tv_sec > start.tv_sec)
+	diff.tv_sec = end.tv_sec - start.tv_sec;
+    else
+	diff.tv_sec = 0;
+
     return diff;
 }
 
