@@ -24,15 +24,17 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: help.c,v 1.8 2002/03/06 19:23:20 martinea Exp $
+ * $Id: help.c,v 1.12 2006/05/25 01:47:14 johnfranks Exp $
  *
  * implements the "help" command in amrecover
  */
 
+#include "amanda.h"
 #include "amrecover.h"
 
 /* print a list of valid commands */
-void help_list P((void))
+void
+help_list(void)
 {
     printf("valid commands are:\n\n");
 
@@ -54,11 +56,13 @@ void help_list P((void))
     printf("mode              - show the method used to extract SMB shares\n");
     printf("pwd               - show cwd on virtual file system\n");
     printf("quit\n");
-    printf("listdisk [diskdevice] - list disks\n");
+    printf("listhost          - list hosts\n");
+    printf("listdisk [diskdevice]              - list disks\n");
     printf("setdate {YYYY-MM-DD|--MM-DD|---DD} - set date of look\n");
-    printf("setdisk diskname [mountpoint] - select disk on dump host\n");
-    printf("sethost host      - select dump host\n");
-    printf("settape [host:][device|default] - select tape server and/or device\n");
-    printf("setmode smb|tar   - select the method used to extract SMB shares\n");
+    printf("        {YYYY-MM-DD-HH-MM-SS}      - set date of look\n");
+    printf("setdisk diskname [mountpoint]      - select disk on dump host\n");
+    printf("sethost host                       - select dump host\n");
+    printf("settape [host:][device|default]    - select tape server and/or device\n");
+    printf("setmode smb|tar                 - select the method used to extract SMB shares\n");
     printf("\n");
 }
