@@ -24,7 +24,7 @@
  * file named AUTHORS, in the root directory of this distribution.
  */
 /*
- * $Id: amlabel.c,v 1.53.2.1 2006/10/03 11:35:52 martinea Exp $
+ * $Id: amlabel.c,v 1.53 2006/07/25 18:27:57 martinea Exp $
  *
  * write an Amanda label on a tape
  */
@@ -102,7 +102,7 @@ main(
 
     erroutput_type = ERR_INTERACTIVE;
 
-    parse_server_conf(argc, argv, &new_argc, &new_argv);
+    parse_conf(argc, argv, &new_argc, &new_argv);
 
     if(new_argc > 1 && strcmp(new_argv[1],"-f") == 0)
 	 force=1;
@@ -461,6 +461,10 @@ main(
 	        error("couldn't write tapelist: %s", strerror(errno));
 		/*NOTREACHED*/
 	    }
+
+            if (have_changer) {
+                changer_label(outslot, label);
+            }
 	} /* write tape list */
 	printf(", done.\n");
     } else {
