@@ -34,11 +34,10 @@
 
 #include "event.h"
 
+#include "amanda.h"
 #include "holding.h"
 #include "server_util.h"
 
-#define MAX_DUMPERS 63
-
 #ifndef GLOBAL
 #define GLOBAL extern
 #endif
